#include <iostream>
#include <stdio.h>
#include <string.h>
#include <string>
#include "ACSS.h"
#include "ExpatHeaders.h"
#include "xmlutil.h"
 
 
 
// Constructor
ACSS::ACSS(const char* xmlstring)
{
    error = 0;

    XML_Parser parser = XML_ParserCreate(NULL);
    XML_SetUserData(parser, this);
    XML_SetElementHandler(parser, startElement, endElement);
    if (!XML_Parse(parser, xmlstring, strlen(xmlstring), 1)) 
        error = XML_GetErrorCode(parser);
    else
        error = IntegrityCheck();

    XML_ParserFree(parser);
}


void ACSS::Dump()
{
	PRINT(seedvalue)
	PRINT(numgroups)
	PRINT(numsacrifices)
    PRINT(studyduration)
    PRINT(tumoronsetprob)
    PRINT(shapeparameter)
    PRINT(tumorlethalityrate)
    PRINT(numdatasets)
    PRINT(testlevel)
	PRINT(testsides)
	PRINT(sessionName)
	PRINT(multiplicity)
    for (int i = 0; i < groups.size(); i++){
        PRINT(groups[i].crsr)
        PRINT(groups[i].DoseMetric)
		PRINT(groups[i].numPerGroup)
	}
	for ( i = 0; i < HazardRate.size(); i++)
        PRINT(HazardRate[i])

    for (int j = 0; j < sacrifices.size(); j++)
    {
        
        PRINT (sacrifices[j].SacrificeTimes)
    }
	for ( j = 0; j < intsacrifices.size(); j++)
    {
        PRINT (intsacrifices[j])
        
    }

}

int ACSS::IntegrityCheck()
{
    if (groups.size() == 0) 
        return ERROR_BAD_NUM_GROUPS;

	if (HazardRate.size() == 0) 
        return ERROR_BAD_HAZARD_RATE;

    if (sacrifices.size() < 0)
        return ERROR_BAD_NUM_SACRIFICES;

    if (studyduration <= 0)
        return ERROR_BAD_STUDY_DURATION;

    if ( tumoronsetprob < 0.0 ||  tumoronsetprob > 1.0)
        return ERROR_BAD_TUMOR_ONSET_PROB;

     

    if (numdatasets <= 0)
        return ERROR_BAD_NUM_DATA_SETS;

    return OK;
}

// XML parser callback
void startElement(void *userData, const char *name, const char **atts)
{
    ACSS* pData = (ACSS*) userData;
	const char * testMe = NULL;

    if (strcmp(name, "acss") == 0)
    {
		pData->seedvalue          = (int) getNumberAttribute(atts, "seed");
		pData->numgroups          = (int) getNumberAttribute(atts, "numgroups");
		pData->numsacrifices      = (int) getNumberAttribute(atts, "numsacrifices");
		pData->studyduration      = (int) getNumberAttribute(atts, "studyduration");
		pData->tumoronsetprob     =       getNumberAttribute(atts, "tumoronsetprob");
		pData->shapeparameter     = (int) getNumberAttribute(atts, "shapeparameter");
		pData->tumorlethalityrate =       getNumberAttribute(atts, "tumorlethalityparameter");
        pData->numdatasets        = (int) getNumberAttribute(atts, "numdatasets");
        pData->testlevel          =       getNumberAttribute(atts, "testlevel");
        pData->testsides          = (int) getNumberAttribute(atts, "testsides");
		testMe = getStringAttribute(atts, "sessionName");
		if (testMe)
            pData->sessionName  =  testMe;
	    testMe = getStringAttribute(atts, "multiplicity");
		if (testMe)
		    pData->multiplicity   =  testMe;
	}
    else if (strcmp(name, "group") == 0)
    {   
		Group g;
        g.crsr        = getNumberAttribute(atts, "crsr");
        g.DoseMetric  = getNumberAttribute(atts, "DoseMetric");
		g.numPerGroup = (int) getNumberAttribute(atts, "NumPerGroup");
        pData->groups.push_back(g);
    }
    else if (strcmp(name, "sacrifice") == 0)
    {
        Sacrifice s;
        s.SacrificeTimes  = (int) getNumberAttribute(atts, "SacrificeTimes");
        pData->sacrifices.push_back(s);
    }
	else if (strcmp(name, "hazardgroup") == 0)
    {
        pData->HazardRate.push_back(getNumberAttribute(atts, "HazardRate"));
    }
	else if (strcmp(name, "interimsacrifices") == 0)
    {
          
        pData->intsacrifices.push_back((int) getNumberAttribute(atts, "numInterimSacrifices"));
     }
}

// composeStruct

void composeStruct(xmlInputDataStruct * MyStruct, ACSS* pData)
{
MyStruct->selectedSeed            = pData->seedvalue;
MyStruct->numberDoseGroups        = pData->numgroups;
MyStruct->numberSacrifices        = pData->numsacrifices;
MyStruct->durationOfStudy         = pData->studyduration;
MyStruct->shapeParameter          = pData->shapeparameter;
MyStruct->tumorLethalityParameter = pData->tumorlethalityrate;
MyStruct->numberDataSets          = pData->numdatasets;
MyStruct->testLevel				  = pData->testlevel;
MyStruct->testSides               = pData->testsides;
MyStruct->tumorOnsetProbability   = pData->tumoronsetprob;
strncpy(MyStruct->sessionName, pData->sessionName.c_str(),49);

for(int i=0; i<pData->groups.size(); i++)
{
    MyStruct->doseLevel[i]                  = pData->groups[i].DoseMetric;
	MyStruct->competingRiskSurvRate[i]      = pData->groups[i].crsr;
	MyStruct->numberAnimalsInGroup[i]       = pData->groups[i].numPerGroup;
}

for( i=0; i<pData->sacrifices.size(); i++)
{
    
	MyStruct->sacrificeTimepoint[i]      = pData->sacrifices[i].SacrificeTimes;
}

for( i=0; i<pData->HazardRate.size(); i++)
{
    MyStruct->hazardRate[i]    = pData->HazardRate[i];
}

 
if(strcmp(pData->multiplicity.c_str(),"Multiple") == 0)
{
	int row = 0;
	int col = 0;
	for (row = 0 ; row< pData->numgroups ; row++)
	{
		for (col = 0; col< (pData->numsacrifices - 1) ; col++)
		{
			i = row*(pData->numsacrifices - 1) + col; 
			MyStruct->numberInterimSacrifices[row][col]    = pData->intsacrifices[i];
		} 
	}
}
return; //;-)
}


//
void initializeXMLStruct(xmlInputDataStruct * MyStruct, ACSS* pData )
{
    MyStruct->doseLevel               = allocate_real_vector(0,pData->numgroups);
    MyStruct->numberAnimalsInGroup    = allocate_integer_vector(0,pData-> numgroups);
    MyStruct->competingRiskSurvRate   = allocate_real_vector(0,pData->numgroups);
    MyStruct->hazardRate              = allocate_real_vector(0,pData->numgroups);
    if(strcmp(pData->multiplicity.c_str(),"Multiple") == 0)
	{
		MyStruct->sacrificeTimepoint      = allocate_integer_vector(0,pData->numsacrifices);
	    MyStruct->numberInterimSacrifices = mem_alloc_mat_int(0,pData->numgroups,0,pData->numsacrifices);
	}
    return;
}

void cleanupXMLStruct(xmlInputDataStruct * MyStruct, ACSS * pData ) /* LHW */
{
    // double *allocate_real_vector(int l, int u)
    // void free_real_vector(double *v, int l)
    free_real_vector(MyStruct->doseLevel, 0);

    // int *allocate_integer_vector(int l, int u)
    // void free_integer_vector(int *v, int l)
    free_integer_vector(MyStruct->numberAnimalsInGroup, 0);
   

    free_real_vector(MyStruct->competingRiskSurvRate, 0);
    free_real_vector(MyStruct->hazardRate,            0);

    // int **mem_alloc_mat_int(int lr, int ur, int lc, int uc)
    // void free_int_matrix(int **m, int lr, int ur, int lc)
	if(strcmp(pData->multiplicity.c_str(),"Multiple") == 0)
	{
		 free_integer_vector(MyStruct->sacrificeTimepoint,   0);	
		free_int_matrix(MyStruct->numberInterimSacrifices,0,pData->numgroups,0);
	}
    return;
}


//
void dumpXMLStruct(xmlInputDataStruct * MyStruct, ACSS* pData )
{
	PRINT(MyStruct->selectedSeed)
	PRINT(MyStruct->numberDoseGroups)
	PRINT(MyStruct->numberSacrifices)
    PRINT(MyStruct->durationOfStudy)
    PRINT(MyStruct->tumorOnsetProbability)
    PRINT(MyStruct->shapeParameter)
    PRINT(MyStruct->tumorLethalityParameter)
    PRINT(MyStruct->numberDataSets)
    PRINT(MyStruct->testLevel)
	PRINT(MyStruct->testSides)
	PRINT(MyStruct->sessionName)

	for (int i = 0; i < pData->groups.size(); i++){
        
        PRINT(MyStruct->doseLevel[i])
	}
    for ( i = 0; i < pData->groups.size(); i++)
	{
	    PRINT(MyStruct->competingRiskSurvRate[i])
	}
	for ( i = 0; i < pData->groups.size(); i++)
	{
	    PRINT(MyStruct->numberAnimalsInGroup[i])
	}
	for ( i = 0; i < pData->sacrifices.size(); i++)
	{
	    PRINT(MyStruct->sacrificeTimepoint[i])
	}
     
    for( i=0; i<pData->HazardRate.size(); i++)
	{
		PRINT(MyStruct->hazardRate[i])
	}

	for(int row=0; row< pData->numgroups ; row++)
	{
       for(int col = 0; col< pData->numsacrifices - 1 ; col++)
	   {
	        
           PRINT(MyStruct->numberInterimSacrifices[row][col]);
	   } 
	}
return; //;-)
}
//
//
void dumpXMLStructToFile(xmlInputDataStruct * MyStruct, ACSS* pData )
{
	FILE * output;
	
	if((output = fopen("output.txt","w")) == NULL)
		perror("Could not create outputfile");
     
	fprintf(output,"selectedSeed : %d\n",MyStruct->selectedSeed);
	fprintf(output,"numberDoseGroups : %d\n", MyStruct->numberDoseGroups);
	fprintf(output,"numberSacrifices : %d\n",MyStruct->numberSacrifices);
    fprintf(output,"durationOfStudy : %d\n", MyStruct->durationOfStudy);
    fprintf(output,"tumorOnsetProbability : %f\n", MyStruct->tumorOnsetProbability);
    fprintf(output,"shapeParameter : %f\n", MyStruct->shapeParameter);
    fprintf(output,"tumorLethalityParameter : %f\n", MyStruct->tumorLethalityParameter);
    fprintf(output,"numberDataSets : %d\n", MyStruct->numberDataSets);
    fprintf(output,"testLevel : %f\n", MyStruct->testLevel);
	fprintf(output,"testSides : %d\n", MyStruct->testSides);
	fprintf(output,"sessionName : %s\n", MyStruct->sessionName);

	for (int i = 0; i < pData->groups.size(); i++){
        
        fprintf(output,"doseLevel[%d] : %f\n",i, MyStruct->doseLevel[i]);
	}
    for ( i = 0; i < pData->groups.size(); i++)
	{
	    fprintf(output,"competingRiskSurvRate[%d] : %f\n", i, MyStruct->competingRiskSurvRate[i]);
	}
	for ( i = 0; i < pData->groups.size(); i++)
	{
	    fprintf(output,"numberAnimalsInGroup[%d] : %d\n", i, MyStruct->numberAnimalsInGroup[i]);
	}
	for ( i = 0; i < pData->sacrifices.size(); i++)
	{
	    fprintf(output,"sacrificeTimepoint[%d] : %d\n", i, MyStruct->sacrificeTimepoint[i]);
	}
     
    for( i=0; i<pData->HazardRate.size(); i++)
	{
		fprintf(output,"hazardRate[%d] : %f\n",i, MyStruct->hazardRate[i]);
	}

	for(int row=0; row< pData->numgroups ; row++)
	{
       for(int col = 0; col< pData->numsacrifices - 1 ; col++)
	   {
	        
           fprintf(output,"numberInterimSacrifices[%d][%d] : %d\n", row, col,MyStruct->numberInterimSacrifices[row][col]);
	   } 
	}
    fclose(output);
}

//
void endElement(void *userData, const char *name)
{

}