#ifndef ACSS_H
#define ACSS_H

#include <vector>
#include <string>
#include "driveP.h"

// Struct elements of the ACSS class
struct Sacrifice
{
    
    int SacrificeTimes;
};

 

struct Group
{
    double crsr;
    double DoseMetric;
	int numPerGroup;
};

 

class ACSS
{
public:
     

    // The number of groups is crsr.size() + 1.
   	std::vector<double> HazardRate;
	std::vector<double> intsacrifices;
    // The total number of sacrifices is sacrifices.size() + 1.
    // We don't track data on the last sacrifice, so it's not included.
    std::vector<Sacrifice> sacrifices;
	std::vector<Group> groups;

	int         seedvalue;
    int		    numgroups;
	int		    numsacrifices;
    int         studyduration;
	int		    shapeparameter;
	double      tumorlethalityrate;
    double      tumoronsetprob;  // background tumor onset probability
    double      testlevel;
	int         testsides;
    int         numdatasets;
	std::string sessionName;
	std::string multiplicity;
    int         error;

    ACSS(const char* xmlstring);
    void Dump();
    int IntegrityCheck();
};

// We define our error codes outside the range of
// the error codes possibly returned by the parser.
#define OK                                          0
#define ERROR_BAD_NUM_GROUPS                        101
#define ERROR_BAD_NUM_PER_GROUP                     102
#define ERROR_BAD_CRSR                              103
#define ERROR_BAD_NUM_SACRIFICES                    104
#define ERROR_BAD_SACRIFICE                         105
#define ERROR_BAD_NUM_ANIMALS_IN_SACRIFICE          106
#define ERROR_BAD_TIME_POINT_IN_SACRIFICE           107
#define ERROR_BAD_HAZARD_RATE                       108
#define ERROR_BAD_END_TIME                          109
#define ERROR_BAD_BTOP                              110
#define ERROR_UNRECOGNIZED_TUMOR_LETHALITY_RATE     111
#define ERROR_BAD_NUM_DATA_SETS                     112
#define ERROR_BAD_DOSE_METRIC						113
#define ERROR_BAD_STUDY_DURATION                    114
#define ERROR_BAD_TUMOR_ONSET_PROB                  115  


#define PRINT(x) std::cout << #x << ": " << x << "\n";

void startElement(void *userData, const char *name, const char **atts);
void endElement(void *userData, const char *name); // no need to implement
void composeStruct(xmlInputDataStruct * MyStruct, ACSS* pData);
void initializeXMLStruct(xmlInputDataStruct * MyStruct, ACSS* pData );
void dumpXMLStruct(xmlInputDataStruct * MyStruct, ACSS* pData);
void cleanupXMLStruct(xmlInputDataStruct * MyStruct, ACSS* pData );
void dumpXMLStructToFile(xmlInputDataStruct * MyStruct, ACSS* pData );
 

#endif