#ifndef DRIVE_H
#define DRIVE_H

#include "cplusplus.h"
#include "driveP.h"

#define delta1	-log(1.0 - pt1)	/* to calculate the time to onset of tumor */
#define xtol	1.0e-4	/* error convergence rate for the bisection method */
#define gamma1	1.0e-4	/* factor for the dist. of RV. T2 and T3 */
#define gamma2	1.0e-16	/* factor for the dist. of RV. T2 and T3 */
 

/* Macro definitions used in  gen_adhoc.c */
#define swap(x,y)	{double t; t = x; x = y; y = t;}
#define order(x,y)	if(x > y) swap(x,y)
#define order2(x,y)		order(x,y)
#define order3(x,y,z)	order2(x,y); order2(x,z); order2(y,z)

/*
 * Typedefs
 */

typedef struct{
        double  sac, t1, tt, t3, die, d_intvl_time;
} animalinfo;
typedef animalinfo *Animalarry;
Animalarry animals;

typedef enum {yes, no}  yes_no;

struct sampleData{
int fixedinterval;
int * numberDead;
int * numDeadWithTumor;
int * numDeadWithoutTumor;
int * numSacrificesWithTumor;
int * numSacrificesWithoutTumor;
double * intervalEndpoints;
//double r_gam; RGN 09/26/2002 to be used in ver 1.1
double dose_level;
};


struct gendataInputStruct{
	int dose_group;
	int N_tot;
	int selectseed;
	int NTP;
    int* sactime;
	int** no_sac_int;
	int rn;
	double* dose_level;
	double* ed;
	double* comp;
	double* tpt1;
	//double* r_gam; RGN 09/26/2002 to be used in ver 1.1
	//double* hazardrt;
	int* N;
	double pt1;
	double delta2;
	double phi2;
	int intvl_flg;
	int fix_intvl;
	int maxintvl;
}; 


struct petotestInputStruct{
	int numGroups;
	int test_side;
	double test_level;
	int maxintvl;
	int* animal;

};

struct averages{
	int numGroups;
	int intvl_flg;
	int NTP;
	int nit;
    int* N;
    int* sactime;
	double* avg_t1;
	double* avg_t3;
	double* avg_leth;
	double* divisor;
	int** d_avg;
	int** a1_avg;
	int** b1_avg; 
	int** a2_avg;
	int** b2_avg;
};

typedef struct sampleData sampleData_t;
typedef struct gendataInputStruct gendataInputStruct_t;
typedef struct petotestInputStruct petotestInputStruct_t;
typedef struct averages averages_t;

/*
 *  Global Variables
 */


/* indicate whether sum of d,a1,da1 is zero or not */
int d_flag, a1_flag, da1_flag, intvl_inc;
/* #death with incidental tumor: a1+a2, No. of death: a1+a2+b1+b2 */
int **no_w_it, **no_dead_it;
int* y_g_sum;
int* n_g_sum; 
/* No. of death with fatal tumor ie. d,  No. of survival animal */
int **no_w_t, **no_surv;
/* y[][j], n[][j] where j = 1..m for fatal case, For poly3 test, #tumors */
int *y_g_sum_ft, *n_g_sum_ft, *y_poly3; 
double pwr_poly3, pwr_ca, pwr_camod;
double pwr_trend_comb, pwr_trend_ft, pwr_trend_inc;
/* actual counter for fatal, incidental, and combined */
double nit_ft, nit_inc, nit_comb, gamma3, f_gam;
/* dose concentration;  For poly3 test #deaths; D[i] for i = 1..g */
double *dose, *gam_poly3, *D, *D_ft, *D_tot; 
double  **K, **K_ft; /* k[i][j] for i = 1..g & j = 1..m */
double **E, **E_ft; /* E[i][j] for i = 1..g & j = 1..m */
double **V_tot, **V_a, **V_b; /* V[i][j] for i = 1..g & j = 1..g */

/*
 * Private Functions
 */
_BEGIN_CPLUSPLUS 

int petotest( const petotestInputStruct_t, int, double);
int gendata( const gendataInputStruct, sampleData_t *, averages_t *, int * );
void init_incidental( int, sampleData* );
void init_fatal( int, int, sampleData* );
void make_D_vec( int, int, int );
void gen_animal_sort( int index[], double temp[], double A[], int N );
void assgn_arry( double *, int *, int );
int  assign_fix_intvl( double *, double *, int, int );
void assgn_arry_ft( double *, int, int, int );
void quicksort( double *, double * );
void roundoff_arry( double *, int );
int det_no_of_intvl_ft( int, int );
int decide_intvl( double [], int );

int cal_expect( int, int, int );
int calc_stat(int cnt, 
			  int group, 
			  double *m_2, 
			  double *tmp_D,
	          double **tmp_V,
			  double z_alpha);
int cal_V_matrix(int, int, int, double * , double * );
int find_psdinv(int, int, int, double);
int perform_poly3_CA(int, int *, int);
int psdinv(double **, int, int, double[]);
double *partition(double *, double *, double );
void GetAverages(const averages_t );
void CalculatePower();
void InitialProcess ( xmlInputDataStruct, gendataInputStruct*, petotestInputStruct *,
					 averages_t * );
void InitializeAverages( averages_t *, const int *, int, int, int, int );
void AllocateDataStructs( sampleData_t **, int, int );
void DeallocateDataStructs( sampleData_t* sampData, int numGroups );
void FinalProcess( int, gendataInputStruct_t, petotestInputStruct_t, averages_t );
double zbrent( double, double, double, double, int); 
 
 
_END_CPLUSPLUS 

#endif /* DRIVE_H */
